package cn.kdan.compdf.config;

import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

/**
 * 发送消息确认
 * @author Bob 2022-11-02
 **/
@Slf4j
@Component
public class RabbitTemplateConfig implements RabbitTemplate.ConfirmCallback,RabbitTemplate.ReturnCallback {

    private final RabbitTemplate rabbitTemplate;

    public RabbitTemplateConfig(RabbitTemplate rabbitTemplate) {
        this.rabbitTemplate = rabbitTemplate;
    }

    @PostConstruct
    public void initRabbitTemplate() {
        rabbitTemplate.setConfirmCallback(this);
        rabbitTemplate.setReturnCallback(this);
    }

    @Override
    public void confirm(CorrelationData correlationData, boolean b, String s) {
        if (!b) {
            // 发送到交换机失败
            log.warn("correlationData -> {}, ack -> {}, cause -> {}", correlationData, b, s);
        }
    }

    @Override
    public void returnedMessage(Message message, int i, String s, String s1, String s2) {
        // todo 发送的消息持久化设置
        // 发送到队列失败:消息主体 message,消息主体 replyCode,描述 replyText,消息使用的交换器,消息使用的路由键
        log.warn("message -> {}, replyCode  -> {}, replyText -> {}, exchange -> {}, routingKey -> {}", message, i, s, s1, s2);
    }
}
